#ifndef _SCENE_H_
#define _SCENE_H_

#include "image.h"
#include "common.h"

typedef struct _voxel {
  int x,y,z;
} Voxel;

typedef struct _scene {
  int *data;
  int xsize,ysize,zsize;
  float dx,dy,dz;
  int *tby, *tbz;
  int maxval, n;
} Scene;

Scene  *CreateScene(int xsize,int ysize,int zsize);
void    DestroyScene(Scene **scn);
Scene  *CopyScene(Scene *scn);
int     VoxelValue(Scene *scn,Voxel v);
bool    ValidVoxel(Scene *scn, int vx, int vy, int vz);
int     MaximumValue3(Scene *scn);
int     MinimumValue3(Scene *scn);
Scene  *ReadScene(char *filename);
void    WriteScene(Scene *scn, char *filename);
Scene  *LinearInterp(Scene *scn,float dx,float dy,float dz);
Scene  *KnnInterp(Scene *scn,float dx,float dy,float dz);
int     KnnInterpValue(Scene *scn, float x, float y, float z);
int     SceneImax(Scene *scn);
void    SetSceneImax(Scene *scn, int val);

int GetXSize(Scene *scn);
int GetYSize(Scene *scn);
int GetZSize(Scene *scn);
float GetDx(Scene *scn);
float GetDy(Scene *scn);
float GetDz(Scene *scn);
void SetDx(Scene *scn, float dx);
void SetDy(Scene *scn, float dy);
void SetDz(Scene *scn, float dz);
void SetVoxelSize(Scene *scn, float dx, float dy, float dz);
int Coord2Voxel(Scene *scn, int x, int y, int z);
int GetVoxelValue(Scene *scn, int voxel);

int ScenesAreEqual(Scene *scn1, Scene *scn2);
void CompareScenes(Scene *scn1, Scene *scn2);
void SetScene(Scene *scn, int value);
void SetLabelScene(Scene *scn, int label, int value);

#define GetVoxel(s,n) ((s)->data[(n)])
#define SetVoxel(s,n,i) (s)->data[(n)]=i
#define SceneLen(s) ((s)->xsize * (s)->ysize * (s)->zsize)
#define VoxelX(s,n) (((n) % (((s)->xsize)*((s)->ysize))) % (s)->xsize)
#define VoxelY(s,n) (((n) % (((s)->xsize)*((s)->ysize))) / (s)->xsize)
#define VoxelZ(s,n) ((n) / (((s)->xsize)*((s)->ysize)))
#define VoxelAddress(s,x,y,z) ((x)+(s)->tby[(y)]+(s)->tbz[(z)])

#endif

