#ifndef RANDOM_WALK_H_
#define RANDOM_WALK_H_

#include "scene.h"
#include "adjacency3.h"
#include "lis.h"

typedef struct LinearSystemStr {
    LIS_MATRIX A;
    LIS_VECTOR x;
    LIS_VECTOR *B;
} *LinearSystem;

typedef struct RandomWalkStr {
    Scene *input, *output, *labels;
    char *output_scene_file;
    int num_labels;
    int num_unlabeled; 
    int maxval;
    AdjRel3 *adj;
    LinearSystem lin_sys;
    double *last_x;
    double *max_prob;
} *RandomWalk;

/*
 * Initializes the Random Walk algorithm (doesn't execute it), returning
 * the RandomWalk file that must be passed to the other functions.
 * input_scene_file: file name of the input scn scene to segment
 * output_scene_file: file name of the output scn file,
 *                    with the labels indexes assigned by the segmentation
 * label_file: scn file with the labels indexes from 1 to n. Voxels that *             will be assigned to one of the labels must have the value 0.  * num_labels: number of labels int the label_file (the 'n' mentioned above).  */
RandomWalk RANDOMWALKinit (char *input_scene_file, char *output_scene_file,
        char *label_file, int num_labels);
/* Frees up the memory on the rw struct */
void RANDOMWALKdestroy (RandomWalk *rw);
/* Executes the Random Walks algorithm with the information stored on rw */
int RANDOMWALKrun (RandomWalk rw);

#endif
